package de.fraunhofer.iem.app.kpi.dto

import de.fraunhofer.iem.app.kpi.enumeration.KpiKind
import de.fraunhofer.iem.app.logger.getLogger

class KpiCalculationDto(
    val kind: KpiKind,
    private val calculationStrategy: de.fraunhofer.iem.app.kpi.strategy.KPICalculationStrategy? = null,
    private var isEmpty: Boolean = true,
    private var value: Int? = null
) {
    private val hierarchyEdges: MutableList<KPIHierarchyEdgeDto> = mutableListOf()
    private val logger = getLogger(javaClass)

    fun hasChildren(): Boolean {
        return hierarchyEdges.isNotEmpty()
    }

    fun getHierarchyEdges(): List<KPIHierarchyEdgeDto> {
        return hierarchyEdges.toList()
    }

    fun isEmpty(): Boolean {
        return isEmpty
    }

    fun calculateKPI() {
        if (calculationStrategy != null) {
            try {
                val valueAndEdges = calculationStrategy.calculateKPI(this.hierarchyEdges)
                this.value = valueAndEdges.first
                this.hierarchyEdges.clear()
                this.hierarchyEdges.addAll(valueAndEdges.second)

                this.isEmpty = false
            } catch (exception: Exception) {
                logger.error("Exception during KPI calculation for $kind with $calculationStrategy")
                this.isEmpty = true
                this.value = null
            }
        } else {
            this.isEmpty = true
        }
    }

    fun getValue(): Int {
        if (value != null && !isEmpty) {
            return value as Int
        } else {
            throw Exception("Value access for empty KPIs not allowed. KpiKind $kind")
        }
    }

    fun addChildKPI(kpiCreateDto: KpiCalculationDto, weight: Double) {
        val hierarchyEdge = KPIHierarchyEdgeDto(this, kpiCreateDto, weight)
        this.hierarchyEdges.add(hierarchyEdge)
    }

    fun toViewModel(): KPITreeResponseDto {
        val children = this.hierarchyEdges.map {
            val child = it.to.toViewModel()
            KPITreeChildResponseDto(kpi = child, plannedWeight = it.plannedWeight, actualWeight = it.actualWeight)
        }

        return this.kind.toViewModel(value = value ?: -1, children = children, isEmpty = isEmpty)
    }
}
